/*
 * Copyright (c) 2011 Aalto University and RWTH Aachen University.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#define _BSD_SOURCE

#include "config.h"
#include "firewall/firewall.c"
#include "firewall/firewall_defines.h"
#include "firewall/midauth.h"
#include "firewall/midauth.c"
#include "modules/midauth/lib/midauth_builder.h"
#include "test_suites.h"
#include "test/mocks.h"

START_TEST(test_hipfw_midauth_add_challenge)
{
    struct hip_fw_context               ctx;
    uint8_t                             midauth_nonce[MIDAUTH_DEFAULT_NONCE_LENGTH];
    const struct hip_challenge_request *challenge = NULL;
    unsigned char                       buf[]     = "\x00\x30\xF9\x16\x00\x88\xFF\xFF\x00\x00\x00\x00\x00\x00\x00\x00\x23\x14\x49\x4E\x00\x00\x00\x00"
                                                    "\x93\x28\x06\x00\x00\x00\x00\x00\x02\x00\x00\x00\x65\x74\x68\x32\x00\x00\x00\x00\x00\x00\x00\x00"
                                                    "\x00\x00\x00\x00\x65\x74\x68\x31\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x08\x00\x01\x00"
                                                    "\x06\x08\x00\x27\x79\x45\x6B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x18\x03\x00\x00\x00\x00\x00\x00"
                                                    "\x45\x00\x03\x18\x00\x00\x40\x00\x3F\x11\x1D\xD3\x0A\x00\x04\x02\x0A\x00\x03\x01\x29\x04\x29\x04"
                                                    "\x03\x04\x00\x00\x00\x00\x00\x00\x3B\x5E\x02\x11\x00\x00\x00\x00\x20\x01\x00\x10\xD5\x79\x52\xA7"
                                                    "\xFA\xC2\x99\x64\xC9\x36\xA9\x40\x20\x01\x00\x19\x9C\x8E\x9A\x92\xD5\x68\xD2\x87\x7F\x3F\x6F\x95"
                                                    "\x01\x01\x00\x0C\x00\x2A\x48\x49\xF3\xAC\x4C\x3F\x25\x09\xCC\x58\x02\x01\x00\xF6\x03\x00\xC0\x33"
                                                    "\xAE\x3C\x0A\x08\xC3\x47\x2B\x46\xB2\xDD\xE7\xFA\x5E\xC7\x5E\x25\xEE\x8B\x4A\xF4\xF2\xEE\xB8\x25"
                                                    "\x7C\x74\x6F\x3C\x1C\x4F\x51\x87\xF3\xF4\x0E\x8E\x88\xDA\xFF\x55\x2D\x51\x4E\xEB\x2F\x4B\x8D\x89"
                                                    "\xE5\xAD\xEA\xD5\x87\x0D\x20\x9C\xF6\x93\x0A\x05\x2A\x29\x52\x05\x91\x84\x70\xF9\xBE\xF2\x87\x04"
                                                    "\x4D\x5C\x7F\xC9\x20\x8D\x76\x22\xEC\xA5\x0D\x6C\x06\xB5\xB2\x30\x76\x01\x52\x4B\xCD\xD8\x41\xAD"
                                                    "\x59\x95\x0B\xE2\xEC\xAF\xC3\x95\xFA\xB1\x3B\xE2\x06\x51\xB0\x4B\x3F\x6C\x61\xDB\xF1\x08\x25\x11"
                                                    "\xBE\xB6\x4B\x39\x6F\x72\x03\x80\x3A\x4A\x86\xFE\xC4\xC0\x9D\x8B\x8E\x88\x99\x06\xF0\x72\x29\xC6"
                                                    "\x66\xA8\x31\x36\x38\x50\x10\x4F\xD5\xF7\xAE\x39\xDD\xB1\x47\x4B\x14\x59\xA5\x21\x1C\x60\x2D\xF0"
                                                    "\xB7\x16\x71\xEA\xA8\x7B\xC5\xD1\x7F\x60\xD2\xE5\x36\xEF\xED\xBC\xE3\x9E\x63\x85\x19\xC5\xE1\x01"
                                                    "\x00\x30\x4E\x9E\x69\x8F\x6E\xFA\xAF\x95\x9F\x55\x78\x25\xE1\x48\x59\x40\x1D\x5A\x14\x5A\xB2\xA3"
                                                    "\x86\x67\x1C\xE9\x80\x5A\x7A\xC0\xE6\xFE\x6D\x96\x2B\xC5\x8C\x54\x20\xAE\x15\xFA\x7B\xEC\xF9\x1C"
                                                    "\x35\xAB\x00\x00\x00\x00\x00\x00\x02\x41\x00\x06\x00\x01\x00\x02\x00\x05\x00\x00\x00\x00\x00\x00"
                                                    "\x02\xC1\x00\xD4\x00\xC8\x10\x08\x02\x02\xFF\x05\x03\x01\x00\x01\xD9\x83\x29\xBE\xF8\x4C\x5A\xCC"
                                                    "\x9B\x71\x0B\x4E\x26\xCE\x2A\x21\x6C\x29\xC0\x2A\x16\x38\xD1\x92\xE3\x1B\x25\x9D\x65\x24\xF5\x83"
                                                    "\x18\x0D\x61\xE7\xA9\x09\x09\x9C\x8F\xBF\xC1\x07\x02\xBF\x62\x0D\x82\x62\x4C\x81\xEB\x3A\xE2\x62"
                                                    "\x19\xEB\x51\x7A\x12\xDF\x24\x37\x5F\x18\xB4\xF1\x28\xD6\x08\xFF\xE0\x6B\x48\x4E\x13\xC1\x61\x1E"
                                                    "\x72\xF1\xD6\x32\xA1\x43\x42\x37\x66\x45\x6F\x4A\x7A\x92\x51\xE0\x98\x1D\x0F\x92\x18\x37\xB7\xCC"
                                                    "\xCC\x84\xF9\x61\x6C\xE8\xF6\xD7\x3D\xF3\x1D\x04\x4F\xD1\x4C\xA8\x78\x63\x73\xAD\x9C\x2B\xBE\x4D"
                                                    "\x25\x1B\xC5\x9C\x5D\x0F\x6C\xC6\x21\x01\xD1\xFA\xAD\x62\x3C\x5E\x17\x46\xF9\x2C\xE8\xC8\xE2\x8A"
                                                    "\xC0\x04\xD6\x72\xBE\x9E\x25\xB6\x25\x3C\x11\x1F\x8A\x9F\xE7\xAC\x8F\xBC\xD3\x76\x36\xEC\x7E\x41"
                                                    "\xB9\xBC\x5D\x8F\xC1\xD9\x82\x99\xEA\x9B\x70\x00\xA0\xEB\x08\x2F\x74\x68\x65\x73\x69\x73\x32\x00"
                                                    "\x0F\xFF\x00\x08\x00\x00\x00\x01\x00\x02\x00\x05\x00\x00\x00\x00\xF0\xC1\x00\xC1\x05\x2D\x6F\xDE"
                                                    "\xF9\xCE\x0F\x79\x4E\xA3\x21\x73\x0E\x84\x22\x55\x0C\xCC\x43\xBD\x73\xD3\xA3\x8E\xE8\xC1\xA3\xB6"
                                                    "\x69\x0C\xF7\x12\xA9\x10\x7E\x56\xED\xA2\x83\x23\x19\x6D\x9C\xF1\x65\x22\xFC\xB1\x3A\x96\xC7\xF3"
                                                    "\x35\x9C\xAF\xC8\x2B\x38\x21\x03\xB7\xAE\x8D\xE3\x2D\x92\x2C\x66\x90\x03\xD9\x44\xC6\xA5\xAF\x4B"
                                                    "\xBB\x65\x3A\x50\x84\x33\x2C\x0B\x3D\x59\xF8\x57\x27\x9C\xB1\xB6\x73\xC5\xEE\xC2\x68\x52\x35\x9C"
                                                    "\x41\xD7\x11\x6E\xEC\xF5\x20\x7D\xB8\xB4\x4E\xDD\x20\x94\xA0\x1B\xD8\x60\x8C\x5F\xC1\xB4\x9C\x3C"
                                                    "\xE8\xA3\x99\x81\x6B\xF9\x0D\xE4\xE6\xD9\x34\xD6\x8A\xB5\xBD\xAA\x66\x43\x13\xE0\x30\x9A\x07\x46"
                                                    "\xCD\x9D\x6F\x63\xFF\x5E\x05\x54\xDC\xA9\x10\x3C\x2D\xDF\x62\xAD\x18\x13\x74\x61\x92\xD3\xF7\x65"
                                                    "\xAD\x34\xB1\x4D\xB5\x65\x62\x8A\xD4\x7C\xBC\xF7\xC1\x29\x0F\x92\xB8\xB3\xC9\x05\x4A\x00\x00";

    use_midauth      = 1;
    mock_ipq         = 1;
    mock_ipq_pkt_len = ARRAY_SIZE(buf);

    HIP_DEBUG("Testing hipfw_midauth_add_challenge with valid inputs \n");

    fail_unless(fw_init_context(&ctx, buf, 4) == 0, NULL);

    fail_unless(hipfw_midauth_add_challenge(&ctx, midauth_nonce) == 1, NULL);
    fail_unless((challenge = hip_get_param(ctx.transport_hdr.hip, HIP_PARAM_CHALLENGE_REQUEST)) != NULL, 0);
    fail_unless(memcmp(challenge->opaque, midauth_nonce, MIDAUTH_DEFAULT_NONCE_LENGTH) == 0, NULL);
    fail_unless(challenge->K >= DEFAULT_DIFFICULTY, NULL);
}
END_TEST

#ifdef HAVE_TCASE_ADD_EXIT_TEST
START_TEST(test_hipfw_midauth_add_challenge_NULL_ctx)
{
    struct hip_fw_context ctx;

    HIP_DEBUG("Testing NULL nonce \n");
    use_midauth = true;
    hipfw_midauth_add_challenge(&ctx, NULL);
}
END_TEST

START_TEST(test_hipfw_midauth_add_challenge_NULL_nonce)
{
    uint8_t nonce[18];

    HIP_DEBUG("Testing NULL ctx \n");
    use_midauth = true;
    hipfw_midauth_add_challenge(NULL, nonce);
}
END_TEST

START_TEST(test_hip_challenge_response_opaque_len_NULL)
{
    HIP_DEBUG("Testing hip_challenge_response_opaque_len on NULL input\n");
    hip_challenge_response_opaque_len(NULL);
}
END_TEST

START_TEST(test_verify_response_NULL_ctx)
{
    struct hip_challenge_response response;
    uint8_t                       midauth_nonce[MIDAUTH_DEFAULT_NONCE_LENGTH];

    verify_response(NULL, &response, midauth_nonce);
}
END_TEST

START_TEST(test_verify_response_NULL_response)
{
    struct hip_fw_context ctx;
    uint8_t               midauth_nonce[MIDAUTH_DEFAULT_NONCE_LENGTH];

    verify_response(&ctx, NULL, midauth_nonce);
}
END_TEST

START_TEST(test_verify_response_NULL_nonce)
{
    struct hip_fw_context         ctx;
    struct hip_challenge_response response;

    verify_response(&ctx, &response, NULL);
}
END_TEST

START_TEST(test_hipfw_midauth_verify_challenge_NULL_ctx)
{
    uint8_t midauth_nonce[] = "\x01\x41\x01\x14\x05\x00\x48\x49\x0b"
                              "\x02\x42\x02\x15\x06\x08\x49\x50\x0d";
    hipfw_midauth_verify_challenge(NULL, midauth_nonce);
}
END_TEST

#endif /* HAVE_TCASE_ADD_EXIT_TEST */

START_TEST(test_hip_challenge_response_opaque_len)
{
    struct hip_challenge_response response;

    HIP_DEBUG("Testing hip_challenge_response_opaque_len \n");

    hip_set_param_contents_len(&response.tlv, 28);
    fail_unless(hip_challenge_response_opaque_len(&response) == 18, NULL);

    hip_set_param_contents_len(&response.tlv, 11);
    fail_unless(hip_challenge_response_opaque_len(&response) == 1, NULL);
}
END_TEST

START_TEST(test_verify_response_no_match)
{
    struct hip_fw_context         ctx;
    struct hip_challenge_response response;
    uint8_t                       midauth_nonce[] = "\x01\x41\x01\x14\x05\x00\x48\x49\x0b"
                                                    "\x02\x42\x02\x15\x06\x08\x49\x50\x0d";

    HIP_DEBUG("Testing verify_response on non-matching inputs\n");

    hip_set_param_contents_len(&response.tlv, 28);
    memcpy(response.opaque, midauth_nonce, MIDAUTH_DEFAULT_NONCE_LENGTH);
    response.opaque[0]++;
    fail_unless(verify_response(&ctx, &response, midauth_nonce) == 3, NULL);
    response.opaque[0]--;

    response.opaque[MIDAUTH_DEFAULT_NONCE_LENGTH - 1]++;
    fail_unless(verify_response(&ctx, &response, midauth_nonce) == 3, NULL);
}
END_TEST

START_TEST(test_verify_response)
{
    int                           i;
    struct hip_common             hip;
    struct hip_fw_context         ctx;
    struct hip_challenge_response response;
    struct puzzle_hash_input      tmp_puzzle;
    uint8_t                       midauth_nonce[] = "\x01\x41\x01\x14\x05\x00\x48\x49\x0b"
                                                    "\x02\x42\x02\x15\x06\x08\x49\x50\x0d";

    HIP_DEBUG("Testing verify_response on valid inputs\n");

    // build context
    ctx.transport_hdr.hip = &hip;
    hip.hits              = in6addr_any;
    hip.hitr              = in6addr_any;

    // build response
    fail_unless(hip_midauth_puzzle_seed(midauth_nonce, MIDAUTH_DEFAULT_NONCE_LENGTH, tmp_puzzle.puzzle) == 0, NULL);
    tmp_puzzle.initiator_hit = in6addr_any;
    tmp_puzzle.responder_hit = in6addr_any;
    hip_set_param_contents_len(&response.tlv, 28);
    memcpy(response.opaque, midauth_nonce, MIDAUTH_DEFAULT_NONCE_LENGTH);
    for (i = 1; i <= 20; i++) {
        HIP_DEBUG("Difficulty: %i \n", i);
        response.K = i;
        fail_unless(hip_solve_puzzle(&tmp_puzzle, response.K) == 0, NULL);
        memcpy(response.J, tmp_puzzle.solution, 8);
        fail_unless(verify_response(&ctx, &response, midauth_nonce) == 1, NULL);
    }
}
END_TEST

START_TEST(test_hipfw_midauth_verify_challenge_NULL_challenge)
{
    struct hip_fw_context ctx;
    ctx.packet_type = HIP_PACKET;
    use_midauth     = true;
    fail_unless(hipfw_midauth_verify_challenge(&ctx, NULL) == 0, NULL);
}
END_TEST

START_TEST(test_hipfw_midauth_verify_challenge)
{
    struct hip_common           *hip;
    struct hip_fw_context        ctx;
    struct hip_challenge_request request;
    struct puzzle_hash_input     tmp_puzzle;
    uint8_t                      midauth_nonce[] = "\x01\x41\x01\x14\x05\x00\x48\x49\x0b"
                                                   "\x02\x42\x02\x15\x06\x08\x49\x50\x0d";

    HIP_DEBUG("Testing verify_response on valid inputs\n");

    // build context
    hip                   = hip_msg_alloc();
    hip->hits             = in6addr_any;
    hip->hitr             = in6addr_any;
    ctx.transport_hdr.hip = hip;

    // build request
    request.K        = 1;
    request.tlv.type = HIP_PARAM_CHALLENGE_REQUEST;
    hip_set_param_contents_len(&request.tlv, 20);
    memcpy(request.opaque, midauth_nonce, 18);

    // build solution
    fail_unless(hip_midauth_puzzle_seed(midauth_nonce, MIDAUTH_DEFAULT_NONCE_LENGTH, tmp_puzzle.puzzle) == 0, NULL);
    tmp_puzzle.initiator_hit = in6addr_any;
    tmp_puzzle.responder_hit = in6addr_any;
    fail_unless(hip_solve_puzzle(&tmp_puzzle, request.K) == 0, NULL);

    // build parameter
    hip_build_param_challenge_response(hip, &request, tmp_puzzle.solution);

    fail_unless(hipfw_midauth_verify_challenge(&ctx, midauth_nonce) == 1, NULL);
}
END_TEST

Suite *firewall_midauth(void)
{
    Suite *s          = suite_create("firewall/midauth");
    TCase *tc_midauth = tcase_create("Midauth");

#ifdef HAVE_TCASE_ADD_EXIT_TEST
    tcase_add_exit_test(tc_midauth, test_hipfw_midauth_add_challenge_NULL_ctx, 1);
    tcase_add_exit_test(tc_midauth, test_hipfw_midauth_add_challenge_NULL_nonce, 1);
    tcase_add_exit_test(tc_midauth, test_hip_challenge_response_opaque_len_NULL, 1);
    tcase_add_exit_test(tc_midauth, test_verify_response_NULL_ctx, 1);
    tcase_add_exit_test(tc_midauth, test_verify_response_NULL_response, 1);
    tcase_add_exit_test(tc_midauth, test_verify_response_NULL_nonce, 1);
    tcase_add_exit_test(tc_midauth, test_hipfw_midauth_verify_challenge_NULL_ctx, 1);
#endif

    tcase_add_test(tc_midauth, test_hipfw_midauth_add_challenge);
    tcase_add_test(tc_midauth, test_hip_challenge_response_opaque_len);
    tcase_add_test(tc_midauth, test_verify_response_no_match);
    tcase_add_test(tc_midauth, test_verify_response);
    tcase_add_test(tc_midauth, test_hipfw_midauth_verify_challenge_NULL_challenge);
    tcase_add_test(tc_midauth, test_hipfw_midauth_verify_challenge);

    suite_add_tcase(s, tc_midauth);

    return s;
}
